
=head1 NAME

Comiclib.pm

=cut

=head1 SYNOPSIS

config file and subroutines for comics

=cut

=head1 DESCRIPTION

This is a library of functions and data structures for the ComicMan web comic 
downloading system.

=cut

=head1 AUTHOR

Jay Scott Raymond

=cut

=head1 BUGS

None known as yet.

=cut

=head1 SEE ALSO

inf-doc.txt

=cut

=head1 COPYRIGHT

Copyright 2006, Jay Scott Raymond. All Rights Reserved

This program is free software. You may copy or
redistribute it under the same terms as Perl itself.

=cut

package Comiclib;
require Exporter;
@ISA=qw(Exporter);
require Cwd;
require LWP::UserAgent;
require Digest::MD5;
require Config::General;
use Class::Struct;
#require Time::localtime;
use Time::localtime;

our @EXPORT=qw(get_inf_list fetch_inf_data missing_fields name_fixup 
               get_webpage get_named_comics parse_filename get_cksum_comics 
               prefix_comic rename_comic make_new_inf write_to_logfile debug_print
               comicman_setup get_chosen_list save_chosen_list update_cm_settings get_cm_data 
               set_cm_data make_user_dir init_cm_data);

=head2 FUNCTION: get_safe_home

Parameters: none.

Returns: string.

The get_safe_home() function returns an untainted copy of the Home 
directory path.

=cut

sub get_safe_home() {
my $num_params=@_;
if ($num_params != 0 ) {
	die "get_safe_home:got $num_params params, incorrect\n";
}
my $new_home;
if ($ENV{'HOME'} =~ /^([-\@\?\/\~\w.]+)$/) {
	$new_home = $1;
} else {
	debug_print(1, "get_safe_home: no match");
}
return $new_home;
}

=head2 DATA STRUCTURE: %comicman_data

A hash containing various configuration variables.

=cut

%comicman_data = (
	version => "0.0.3",
	start_dir => Cwd::cwd,
	this_year => localtime->year() + 1900,
	this_month => localtime->mon() +1,
	this_day => localtime->mday(),
	long_date => localtime->year() + 1900 . sprintf( "%02d" , localtime->mon() +1 ) . sprintf( "%02d" , localtime->mday() ),
	short_date => substr(localtime->year() + 1900,2,2) . sprintf( "%02d" , localtime->mon() +1 ) . sprintf( "%02d" , localtime->mday() ),
	user_conf_dir => get_safe_home() . "/.comicman",
	user_comics_dir => "comics",
	user_conf_file => "comicman.conf",
	user_base_dir => get_safe_home() . "/.comicman",
	dload_data_dir =>  get_safe_home() . "/.comicman/dat",
	inf_files_dir =>  get_safe_home() . "/.comicman/inf",
	sysinf_files_dir => "/usr/share/comicman", # this is adjusted by install.pl
	network_timeout => 10,
        target_dir => get_safe_home(),
	dload_log_file => 'comics' . (localtime->year() + 1900 . sprintf( "%02d" , localtime->mon() +1 ) . sprintf( "%02d" , localtime->mday() )) . '.txt',
	log_setting => 0, # 0: logging off, 1: debug_print goes to logfile instead of STDOUT
	debug_level => 0,
        log_range => [0..1],
        debug_range => [0..4],
        timeout_range => [5..30],
);

#=head2 DATA STRUCTURE: struct widget_info

#A Class::Struct for holding the widget data for inf file editing GUI

#=cut

#struct widget_info => { # definition for widget_info
#	field_name	=> '$',		# the name of the data field in an inf file
#	widget_type	=> '$',		# widget type to use in a GUI
#	widget_label	=> '$'		# label to use in a GUI
#};

=head2 FUNCTION: write_to_logfile

Parameter 0: log message

Returns: none.

The write_to_logfile() function writes a log message to the file defined
in $comicman_data{dload_log_file}.

=cut

sub write_to_logfile($) {
my $num_params=@_;
if ($num_params != 1 ) {
	die "write_to_logfile:got $num_params params, incorrect\n";
	return -1;
}
my ($message)=@_;
open LOG, ">>$comicman_data{dload_log_file}"
	or die "Cannot open logfile: $!";
print LOG $message;
close LOG;
}

=head2 FUNCTION: debug_print

Parameter 0: the debug level setting (0: no output, 1: failure, 2: serious problem, 
3: minor problem, 4: trace mode)

Parameter 1: The debug message.

Returns: none.

The debug_print() function prints with multiple debug levels to regulate 
the amount of output.

=cut

sub debug_print($$) {
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 2 ) {
	die "debug_print:got $num_params params, incorrect\n";
	return -1;
}
my ($debug_setting,$message)=@_;
if($comicman_data{debug_level} == 0) {
	return 0;
}
if($comicman_data{debug_level} >= $debug_setting) {
	if ($comicman_data{log_setting} ==0) {
		print $message;
	} else {
		write_to_logfile($message);
	}
}
return 0;
}

=head2 FUNCTION: get_cm_data

Parameter 0: the member of comicman_data to get

Returns: The requested data item or 0 on error.

The get_cm_data() function is an accessor for the comicman_data hash.

=cut

sub get_cm_data($) {
my $num_params=@_;
if ($num_params != 1 ) {
	die "get_cm_data:got $num_params params, incorrect\n";
}
my ($member)=@_;
if( exists($comicman_data{$member}) ) {
	return $comicman_data{$member};
} else {
	return 0;
}
}

=head2 FUNCTION: set_cm_data

Parameter 0: the member of comicman_data to set

Parameter 1: what to set the member to

Returns: 1 = success, 0 = failure.

The set_cm_data() function is an accessor for the comicman_data hash.

=cut

sub set_cm_data($$) {
my $num_params=@_;
if ($num_params != 2 ) {
	die "set_cm_data:got $num_params params, incorrect\n";
}
my ($member,$setting)=@_;
if( exists($comicman_data{$member}) ) {
	$comicman_data{$member} = $setting;
	return 1;
} else {
	return 0;
}
}

=head2 FUNCTION: make_user_dir

Parameters: none

Returns: 0: failure
         1: success

The make_user_dir() function sets up a new user's data directories 

=cut

sub make_user_dir(){
debug_print(4, "in make_user_dir\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 0 ) {
	die "make_user_dir:got $num_params params, incorrect\n";
	return 0;
}
my $return_value;
my $success;
my $home_dir = get_safe_home();

# setup user directories
if( !-e $comicman_data{user_base_dir} ) {
	$return_value=mkdir($comicman_data{user_base_dir});
	if(!$return_value) {
                debug_print(3, "make_user_dir: Couldn't create: $comicman_data{user_base_dir}\n");
		return 1;
	}
}
if( !-e $comicman_data{dload_data_dir} ) {
	$return_value=mkdir($comicman_data{dload_data_dir});
	if(!$return_value) {
                debug_print(3, "make_user_dir: Couldn't create: $comicman_data{dload_data_dir}\n");
		return 1;
	}
}
if( !-e $comicman_data{inf_files_dir} ) {
	$return_value=mkdir($comicman_data{inf_files_dir});
	if(!$return_value) {
                debug_print(3, "make_user_dir: Couldn't create: $comicman_data{inf_files_dir}\n");
		return 1;
	}
}
# make a default user conf file
if ( !-e  "$comicman_data{user_conf_dir}/$comicman_data{user_conf_file}" ) {
	$success = open(CONF, ">$comicman_data{user_conf_dir}/$comicman_data{user_conf_file}" );
	if ( $success ) {
		print CONF <<ENDC;
#log_setting = 0
#debug_level = 1
#network_timeout = 10
ENDC
#target_dir = $home_dir
		undef $success;
		close(CONF);
	} else {
		debug_print(3, "make_user_dir: open of $comicman_data{user_conf_dir}/$comicman_data{user_conf_file} failed\n");
                return 1;
	}
}
return 0;
}

=head2 FUNCTION: comicman_setup

Parameters: none

Returns: none

The comicman_setup() function sets up a new user's data directories via make_user_dir and
which comics to download

=cut

sub comicman_setup() {
debug_print(4, "in comicman_setup\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 0 ) {
	die "comicman_setup:got $num_params params, incorrect\n";
	return 0;
}
my $return_value;
my $inf_file;
my @sys_inf_list;
my @user_inf_list;
my @user_infs;
my $config;
my $file_path;
my %inf_data;
my $answer;
my $success;
# setup user directories
$success = make_user_dir();
if ( $success == 1 ) {
     debug_print(3, "comicman_setup: make_user_dir failed\n");
}

# get list of available infs
@sys_inf_list=get_inf_list($comicman_data{sysinf_files_dir});
@user_inf_list=get_inf_list($comicman_data{inf_files_dir});
# remove items in userInfList that duplicate sysInfList
foreach $inf_file (@sys_inf_list) {
	@user_inf_list = grep(!/$inf_file/,@user_inf_list);
}
# if any left inf userInfList, add them to sysInfList
if (scalar(@user_inf_list) > 0) {
	foreach $inf_file (@user_inf_list) {
		push(@sys_inf_list,$inf_file)
	}
}
@sys_inf_list=sort (@sys_inf_list);
# let the user choose their comics
print <<EOD;
What follows is a list of available comic titles. After each one type \"y\" if
you want that title and type anything else if you don\'t.
EOD
foreach $inf_file (@sys_inf_list) {
	$file_path="$comicman_data{sysinf_files_dir}" . "/$inf_file";
	if (!-e $file_path) {
		$file_path="$comicman_data{inf_files_dir}" . "/$inf_file";
		if (!-e $file_path) {
			debug_print(1, "comicman_setup: $inf_file not found\n");
		}
	}
	$config = new Config::General((-ConfigFile => $file_path , -AutoLaunder => 1));
	if ( defined($config) ) {
		%inf_data = $config->getall;
		undef $config;
	} else {
		$inf_data{fail}=1;
		debug_print(1, "comicman_setup: new Config failed\n");
	}
	print "$inf_data{comicTitle}: ";
	$answer=<STDIN>;
	chomp($answer);
	if($answer eq "y") {
		push(@user_infs,$inf_file);
	}
}
if (scalar(@user_infs) > 0) {
	save_chosen_list(\@user_infs);
}
}

=head2 FUNCTION: update_cm_settings

Parameter 0: Path to config file

Parameter 1: Reference to a list of updatable fields

Returns: 1=success, 0=failure.

The update_cm_settings() function updates %comicman_data with settings from a file.

=cut

sub update_cm_settings($@) {
debug_print(4, "in update_cm_settings\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 2 ) {
	die "check_cm_settings:got $num_params params, incorrect\n";
	return -1;
}
my ($file_path,$allowed_fields_ref)=@_;
# dereference array parameter
my @allowed_fields = @$allowed_fields_ref;
my %inf_data;
my $key;
my $value;
my $config;
my $return_value=1;

# Check for existence of the file.
if ( !-e $file_path ) {
	# not a failure - the files don't need to be there
	debug_print(3,"update_cm_settings:\n$file_path\ndoesn't exist.\n");
	return $return_value;
}
$config = new Config::General((-ConfigFile => $file_path , -AutoLaunder => 1));
if ( defined($config) ) {
	%inf_data = $config->getall;
	undef $config;
	while ( ($key,$value) = each %inf_data ) {
		chomp $inf_data{$key};
		if (scalar(grep(m#$key#,@allowed_fields)) eq 0) {
			debug_print(2,"update_cm_settings: $key not configurable.\n");
		} else {
			$return_value = set_cm_data($key,$value);
			if ($return_value == 0) {
				debug_print(1,"update_cm_settings: unknown key: $key. \n");
			}
		}
	}
} else {
	debug_print(1,"update_cm_settings: could not use config file.\n");
	$return_value = 0;
}

return $return_value;
}

=head2 FUNCTION: init_cm_data

Parameters: none

Returns: 0: failure
         1: success

The init_cm_data() function initializes the comicman_data hash with the user settings

=cut

sub init_cm_data(){
debug_print(4, "in init_cm_data\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 0 ) {
	die "init_cm_data:got $num_params params, incorrect\n";
	return 0;
}
my @sys_conf_fields=('sysinf_files_dir', 'log_setting', 'debug_level', 'network_timeout');
my @user_conf_fields=('log_setting', 'debug_level', 'network_timeout', 'target_dir');

# update from user config file
$user_conf = get_cm_data('user_conf_dir') . "/" . get_cm_data('user_conf_file');
if ( update_cm_settings($user_conf, \@user_conf_fields) == 0 ) {
	debug_print(1, "init_cm_data: Problem with: $user_conf. Please check it. Quitting.\n");
	return 0;
}
return 1;
}

=head2 FUNCTION: get_webpage

Parameter 0: URL of webpage to download

Returns: The text of the webpage.

The get_webpage() function gets the text of a webpage.

=cut

sub get_webpage($) {
debug_print(4, "in get_webpage\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 1 ) {
	die "get_webpage:got $num_params params, incorrect\n";
	return -1;
}
my ($comic_url)=@_;
my $ua;
my $req;
my $res;
my $webpage;
my $status;
# get the webpage
$ua = LWP::UserAgent->new;
$ua->agent("Comicman/$comicman_data{version}");
$ua->timeout($comicman_data{network_timeout});
# Create a request
$req = HTTP::Request->new(GET => $comic_url);
# Pass request to the user agent and get a response back
$res = $ua->request($req);
if ($res->is_success) {
	$webpage = $res->content;
	return $webpage;
} else {
	$status=$res->status_line;
	debug_print(3,"get_webpage: page download failed\n");
	debug_print(3,"get_webpage: $status\n");
	return -1; 	# failure
}
}

=head2 FUNCTION: get_old_data

Parameter 0: data filename

Returns: array of last downloaded comics

The get_old_data() function gets data of the last comic downloaded.

=cut

sub get_old_data($) {
debug_print(4, "in get_old_data\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 1 ) {
	die "get_old_data:got $num_params params, incorrect\n";
	return 0;
}
my ($data_file)=@_;
my @last_comic;
my @chomp_comics;
my $comic;
my $success;
my $file_path;

$file_path = $comicman_data{dload_data_dir} . "/" . $data_file;
if ( !(-e $file_path) ) {
	$last_comic[0] = "bogus";
	debug_print(4, "get_old_data: $file_path not found\n");
} else {
	$success = open(DATGET, "<$file_path" );
	if ( $success ) {
		@last_comic = <DATGET>;
		undef $success;
		close(DATGET);
	} else {
		debug_print(3, "get_old_data: open of $file_path failed\n");
		die "get_old_data: open of $file_path failed\n";
	}
}
# clear trailing cr-lf
foreach $comic (@last_comic) {
	chomp $comic;
	debug_print(4, "get_old_data: data: $comic\n");
	push(@chomp_comics, $comic);
}
return @chomp_comics;
}

=head2 FUNCTION: parse_filename

Parameter 0: webpage text

Parameter 1: regular expression to parse the webpage with

Returns: array of filenames

The parse_filename() function parses a webpage using the provided regEx.

=cut

sub parse_filename($$) {
debug_print(4, "in parse_filename\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 2 ) {
	die "parse_filename:got $num_params params, incorrect\n";
	return 0;
}
my ($web_page,$reg_ex_string)=@_;
my @images=$web_page =~ m%$reg_ex_string%g;
my @fixed_images;
my $comic_name;
my $prev;
my @out;
my $array_size;
$array_size = @images;
if ( $array_size == 0 ){
	debug_print(2,"parse_filename:before untaint: no filenames found\n");
	return @images;
}
foreach $comic_name (@images) {
	if ($comic_name =~ /^([-\@\?\=\&\_\%\~\w.]+)$/) {
		$comic_name = $1; 			# $data now untainted
		push (@fixed_images, $comic_name);
#		print "found image: $comic_name\n";
	} else {
		debug_print(1, "parse_filename: $comic_name: failed to untaint\n");
	}
}
@fixed_images = sort @fixed_images;
$array_size = @fixed_images;
if ( $array_size == 0 ){
	debug_print(2,"parse_filename: no filenames found\n");
	return @fixed_images;
}
# remove duplicates
$prev = "not equal to $fixed_images[0]";
@out = grep($_ ne $prev && ($prev = $_, 1), @fixed_images);
return @out;
}

=head2 FUNCTION: name_fixup

Parameter 0: search part of regEx

Parameter 1: replace part of regEx

Parameter 2: reference to array of filenames

Returns: array of filenames

The name_fixup() function performs adjustments to an array of filenames with a search-replace regEx.

=cut

sub name_fixup($@) {
debug_print(4, "in name_fixup\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 3 ) {
	die "name_fixup:got $num_params params, incorrect\n";
	return 0;
}
# put parameters into local variables
my ($reg_ex_search,$reg_ex_replace,$comics_ref)=@_;
# dereference array parameter
my @comics=@$comics_ref;
my @new_comics;
my $comic;

foreach $comic (@comics) {
	$comic =~ s/$reg_ex_search/$reg_ex_replace/;
	push(@new_comics, $comic);
}
return @new_comics;
}

=head2 FUNCTION: update_comics_file

Parameter 0: datfile name

Parameter 1: reference to an array of comic identifier(s) (could  be filenames, 
numbers or checksums)

Returns: 1=success, 0=failure

The update_comics_file() function updates the data file for a comic.

=cut

sub update_comics_file ($@){
debug_print(4, "in update_comics_file\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 2 ) {
	die "update_comics_file:got $num_params params, incorrect\n";
	return 0;
}
# put parameters into local variables
my ($data_file,$comics_ref)=@_;
# dereference array parameter
my @comics=@$comics_ref;
my $success;
my $file_path;
my $item;

$file_path = $comicman_data{dload_data_dir} . "/" . $data_file;
debug_print(4, "update_data_file: filepath=$file_path\n");
$success = open(DATGET, ">$file_path");
if ( $success ) {
	foreach $item (@comics) {
		print DATGET $item . "\n";
	}
	undef $success;
	close(DATGET);
	return 1;
} else {
	debug_print(1, "update_data_file: open of $file_path failed\n");
	return 0;
}
}

=head2 FUNCTION: prefix_comic

Parameter 0: prefix

Parameter 1: reference to an array of comic(s)

Returns: none

The prefix_comic() function adds a prefix to the comic name(s).

=cut

sub prefix_comic($@) {
debug_print(4, "in prefix_comic\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 2 ) {
	die "prefix_comic:got $num_params params, incorrect\n";
}
# put parameters into local variables
my ($prefix_text,$comics_list_ref) = @_;
my @comics_list = @$comics_list_ref;
my $comic_name;

# rename comics
foreach $comic_name (@comics_list) {
	if ( -e $comic_name ) {
		rename "$comic_name" , "$prefix_text" . "$comic_name";
	}
}
}

=head2 FUNCTION: rename_comic

Parameter 0: text or regEx to be replaced

Parameter 1: replacement text

Parameter 2: reference to an array of comic(s) 

Returns: none

Replacement codes:	#orig# = original filename

The rename_comic() function renames the comic(s).

=cut

sub rename_comic($$@) {
debug_print(4, "in rename_comic\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 3 ) {
	die "rename_comic:got $num_params params, incorrect\n";
}
# put parameters into local variables
my ($source,$replacement,$comics_list_ref) = @_;
my @comics_list = @$comics_list_ref;
my $numComics = @comics_list;
my $comic_name;
my $new_name;
my $incr=0;
my $trans_replace;

# rename comics
foreach $comic_name (@comics_list) {
	if ($comic_name =~ /^([-\@\?\=\&\_\~\w.]+)$/) {
			$comic_name = $1; 			# $data now untainted
	}
	$new_name=$comic_name;
	$trans_replace=$replacement;
	$trans_replace=~s/%orig%/$comic_name/;
	$new_name=~s/$source/$trans_replace/g;

	if ($new_name =~ /^([-\@\?\=\&\_\~\w.]+)$/) {
			$new_name = $1; 			# $data now untainted
	}
	if ( -e $comic_name ) {
		rename ($comic_name , $new_name);
	}
}
}

=head2 FUNCTION: get_named_comics

Parameter 0: Referer URL

Parameter 1: URL to append comic names to

Parameter 2: Data file to update if new comic downloaded 

Parameter 3: reference to an Array of new comic data

Returns: none

The get_named_comics() function gets the named new comic(s), if any.

=cut

sub get_named_comics($$$@) {
#sub get_refered_comics($$$@) {
debug_print(4, "in get_named_comics\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 4 ) {
	die "get_named_comics:got $num_params params, incorrect\n";
}
# put parameters into local variables
my ($referrer,$base_url,$data_file,$comics_ref) = @_;
my @comics = @$comics_ref;
my $comic_file;
my $success;
my $old_comic;
my @old_data;
my @new_data;
my $new_files = 0;
my $iter;
my $ua;
my $res;
my $resmsg;

$ua = LWP::UserAgent->new;
$ua->agent("Comicman/$comicman_data{version}");
$ua->timeout($comicman_data{network_timeout});
# we need this bit or keenspace will give us a bogus file
if ( $referrer ne "none" ) {
	$ua->default_headers->push_header('referer' => $referrer);
}
@old_data = get_old_data( $data_file );
foreach $comic_file (@comics) {
	debug_print(4, "get_named_comics:comicfile:$comic_file\n");
	if ( length($comic_file) < 4 ) {
		debug_print(1, "get_named_comics: failed to parse comic name: $comic_file\n"); 
	}
	if( !(acquired_data($comic_file,\@old_data)) ) {
		debug_print(4, "get_named_comics: $comic_file not aquired\n");
		$file_url = $base_url . $comic_file;
		if ( !(-e $comic_file)) {
			$res = $ua->mirror($file_url,$comic_file);
			if ($res->is_success) {
				debug_print(4, "get_named_comics: mirror successfull\n");
			} else {
				debug_print(1,"get_named_comics: mirror failed\n");
				$resmsg=$res->message();
				debug_print(1, "mirror error: $resmsg\n");
			}
			if ( -e $comic_file ) {
				push(@new_data,$comic_file);
				$new_files++;
			} else {
				debug_print(1, "get_named_comics: comic doesn't exist\n");
			}
		}
	}
	else {
		debug_print(4, "get_named_comics:$comic_file previously acquired\n");
                # this is to kepp from re-downloading old comics
                push(@new_data,$comic_file);
		$found_any = 1;
	}
}
if ( $new_files > 0 ) {
	# only update if we found new comics
	update_comics_file($data_file, \@new_data);
}
}

=head2 FUNCTION: get_cksum_comics

Parameter 0: Referer URL

Parameter 1: URL to append comic names to

Parameter 2: Data file to update if new comic downloaded 

Parameter 3: reference to an Array of new comic data

Returns: none

The get_cksum_comics() function gets the checksumed new comic(s), if any.

=cut

sub get_cksum_comics($$$@) {
debug_print(4, "in get_cksum_comics\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 4 ) {
	die "get_cksum_comics:got $num_params params, incorrect\n";
}
# put parameters into local variables
my ($referrer,$base_url,$data_file,$comics_ref) = @_;
# dereference the array parameter
my @comics = @$comics_ref;
my $comic_file;
my $success;
my $old_comic;
my @old_data;
my @new_data;
my $new_files = 0;
my $ua;
my $res;

$ua = LWP::UserAgent->new;
$ua->agent("Comicman/$comicman_data{version}");
$ua->timeout($comicman_data{network_timeout});
if ( $referrer ne "none" ) {
	$ua->default_headers->push_header('referer' => $referrer);
}

@old_data = get_old_data( $data_file );
my  $md5_obj = Digest::MD5->new;
my $checksum;
foreach $comic_file (@comics) {
	if ( length($comic_file) < 4 ) {
		debug_print(1, "get_cksum_comics: failed to parse comic name: $comic_file\n");
	}
	$file_url = $base_url . $comic_file;
	if ( !(-e $comic_file)) {
		$res = $ua->mirror($file_url,$comic_file);
		if ($res->is_success) {
			debug_print(4, "get_cksum_comics: mirror successful\n");
		} else {
			debug_print(1,"get_cksum_comics: mirror failed\n");
			$resmsg=$res->message();
			debug_print(1, "mirror error: $resmsg\n");
		}
		if ( -e $comic_file ) {
			$success = open DATFILE, "<$comic_file";
			if ($success) {
				binmode(DATFILE);
				$md5_obj->addfile(*DATFILE);
				$checksum = $md5_obj->hexdigest;
				close(DATFILE);
				# if *any* new files are found, we want to store *all* today's checksums
				push(@new_data,$checksum); 
				if( !acquired_data($checksum, \@old_data) ) {
#					push(@new_data,$checksum);
					$new_files++;
				} else {
					unlink $comic_file;
				}
			} else {
				debug_print(1, "get_cksum_comics: file open failed: $comic_file\n");
			}
		} else {
			debug_print(1, "get_cksum_comics: comic doesn't exist\n");
		}
	}

}
if ( $new_files gt 0 ) {
	# only update if we found new comics
	update_comics_file($data_file, \@new_data);
}
}

=head2 FUNCTION: acquired_data

Parameter 0: comic id

Parameter 1: reference to an array of old comic ids

Returns: 1=found, 0=not found

The acquired_data() function checks if a comic id is in the dat file.

=cut

sub acquired_data($@) {
debug_print(4, "in acquired_data\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 2 ) {
	die "acquired_data:got $num_params params, incorrect\n";
}
# put parameters into local variables
my ($comic_id,$old_id_ref) = @_;
# dereference the array parameter
my @old_data = @$old_id_ref;
my $old_id;
my $found_it = 0;

foreach $old_id (@old_data) {
	if($comic_id eq $old_id) {
		$found_it = 1;
	}
}	
return $found_it;
}

=head2 FUNCTION: fetch_inf_data

Parameter 0: inf file name

Returns: comic data hash

The fetch_inf_data() function gets a comic's inf data.

=cut

sub fetch_inf_data($) {
debug_print(4, "in fetch_inf_data\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 1 ) {
	die "fetch_inf_data:got $num_params params, incorrect\n";
}
# put parameters into local variables
my ($inf_name) = @_;

my $file_path;
my %inf_data;
my $key;
my $value;

$file_path="$comicman_data{sysinf_files_dir}" . "/$inf_name";
if (!-e $file_path) {
	$file_path="$comicman_data{inf_files_dir}" . "/$inf_name";
	if (!-e $file_path) {
		$inf_data{fail}=1;
		debug_print(1, "fetch_inf_data: $inf_name not found\n");
		return %inf_data;
	}
}
my $config = new Config::General((-ConfigFile => $file_path , -AutoLaunder => 1));
if ( defined($config) ) {
	%inf_data = $config->getall;
	while ( ($key,$value) = each %inf_data ) {
		chomp $inf_data{$key};
		$value=~s/%thisYear%/$comicman_data{this_year}/;
		$value=~s/%thisMonth%/$comicman_data{this_month}/;
		$value=~s/%thisDay%/$comicman_data{this_day}/;
		$value=~s/%ldate%/$comicman_data{long_date}/;
		$value=~s/%sdate%/$comicman_data{short_date}/;
		$inf_data{$key}=$value;
		if ($key eq "regEx" ) {
			$inf_data{$key}=qr{$value}
		}
	}
}
else
{
	$inf_data{fail}=1;
	debug_print(1, "fetch_inf_data: new Config failed\n");
}
return %inf_data;
}

=head2 FUNCTION: get_inf_list

Parameter 0: inf directory

Returns: array of filenames

The get_inf_list() function gets a list of inf files.

=cut

sub get_inf_list($) {
debug_print(4, "in get_inf_list\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 1 ) {
	die "get_inf_list:got $num_params params, incorrect\n";
}
my ($inf_dir)=@_;
my $inf_file;
my @inf_list;

opendir INFH, $inf_dir;# or die "Cannot open $inf_dir: $!";  # bug? - need more checking here
while ($inf_file = readdir INFH) {
	next unless $inf_file =~ /\.inf$/;
	push(@inf_list,$inf_file);
}
closedir INFH;
@inf_list = sort @inf_list;
foreach $inf_file (@inf_list) {
	if ($inf_file =~ /^([-\@\?\=\~\w.]+)$/) {
		$inf_file = $1; 			# $data now untainted
	}
}
return @inf_list;
}

=head2 FUNCTION: get_chosen_list

Parameters: none

Returns: array of filenames

The get_chosen_list() function gets a list of chosen inf files.

=cut

sub get_chosen_list() {
debug_print(4, "in get_chosen_list\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 0 ) {
	die "get_chosen_list:got $num_params params, incorrect\n";
}
my @chosen_infs;
my $confFile = $comicman_data{user_conf_dir} . '/' . $comicman_data{user_comics_dir};
my $inf;

if (-e $confFile ) {
	open CONF, "<$confFile"
		or die "Cannot open $confFile: $!";
@chosen_infs = <CONF>;
}

debug_print(4, "get_chosen_list: number of infs: ". scalar(@chosen_infs) . "\n");
foreach $inf (@chosen_infs) {
	chomp $inf;
	if($inf=~/^([-\@\?\=\~\w.]+)$/) {
		$inf = $1;  # $data now untainted
	}
}
return @chosen_infs;
}

=head2 FUNCTION: save_chosen_list

Parameter 0: a reference to an array of filenames

Returns: none

The save_chosen_list() function sets a list of chosen inf files.

=cut

sub save_chosen_list(@) {
debug_print(4, "in save_chosen_list\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 1 ) {
	die "save_chosen_list:got $num_params params, incorrect\n";
}
my ($chosen_infs_ref)=@_;
my @chosen_infs =@$chosen_infs_ref;
my $confFile = $comicman_data{user_conf_dir} . '/' . $comicman_data{user_comics_dir};
my $inf;

open CONF, ">$confFile"
	or die "Cannot open $confFile: $!";

foreach $inf (@chosen_infs) {
	print CONF "$inf\n";
}
close CONF;
}

=head2 FUNCTION: missing_fields

Parameter 0: array reference of required field names

Parameter 1: hash reference from inf file

Returns: 1=missing, 0=OK

The missing_fields() function checks inf data for required fields.

=cut

sub missing_fields(@%) {
debug_print(4, "in missing_fields\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 2 ) {
	die "missing_fields:got $num_params params, incorrect\n";
}
my ($fields_ref,$data_ref)=@_;
my @r_fields = @$fields_ref;
my %inf_data=%$data_ref;
my $test_field;
foreach $test_field (@r_fields) {
	if ( !exists($inf_data{$test_field}) ) {
		return 1;
	}
}
return 0;
}

=head2 FUNCTION: make_new_inf

Parameters: none

Returns: none

The make_new_inf() function makes a new inf file with data from the user.

=cut

sub make_new_inf() {
debug_print(4, "in make_new_inf\n");
# verify correct number of parameters
my $num_params=@_;
if ($num_params != 0 ) {
	die "make_new_inf:got $num_params params, incorrect\n";
}
my $config;
my %inf_data;
my $line;
my $line2;
my $inf_name;
my $key;
my $value;

# set the file version
$inf_data{infVersion}="0.0.1";
print "Making new inf file\n";

# get the inf file name
print "Full inf file name? ";
chomp($line = <STDIN>);
if ( !($line=~m%[a-zA-Z0-9\-]*\.inf%)) {
	print "bad filename, start over\n";
	exit;
}
$inf_name = $line;

# Get the comic title
print "Comic Title? ";
chomp($line = <STDIN>);
$inf_data{comicTitle}="$line";

# Get the dat file name
print "Data file name? ";
chomp($line = <STDIN>);
if ( !($line=~m%[a-zA-Z0-9\-]*\.dat%)) {
	print "bad filename, start over\n";
	exit;
}
$inf_data{datFile}="$line";

# Get the pageURL
print "Page URL? ";
chomp($line = <STDIN>);
if ( !($line=~m#^http://[a-zA-Z0-9\~\%\/\.-]*#)) {
	print "malformed URL, start over.\n";
	exit;
}
$inf_data{pageURL}="$line";

# Get the comicURL
print "Comic URL? ";
chomp($line = <STDIN>);
if ( !($line=~m#^http://[a-zA-Z0-9\~\%\/\.-]*#)) {
	print "malformed URL, start over.\n";
	exit;
}
$inf_data{comicURL}="$line";

# Get the regEx
print "Regular Expression? ";
chomp($line = <STDIN>);
$inf_data{regEx}="$line";

# Get the comic type
print "Comic type (1=named,2=checksum)? ";
chomp($line = <STDIN>);
if ($line eq "1") {
	$inf_data{comicType}="named";
} elsif ($line eq "2") {
	$inf_data{comicType}="checksum";
} else {
	print "bad response, start over\n";
	exit;
}

## optional data
# Get the renaming options
print "Comic renaming (1=prefix,2=full,3=none)? ";
chomp($line = <STDIN>);
if($line eq "1") {
	$inf_data{renameIt}="prefix";
	print "Prefix to use? ";
	chomp($line2 = <STDIN>);
	$inf_data{prefix}="$line2";
} elsif ($line eq "2") {
	$inf_data{renameIt}="full";
	print "Rename schema? ";
	chomp($line2 = <STDIN>);
	$inf_data{nameSchema}="$line2";
	print "New name? ";
	chomp($line2 = <STDIN>);
	$inf_data{newName}="$line2";
} elsif ($line eq "3") {
	print "no renaming\n";
} else {
	print "bad response, start over\n";
	exit;
}

# Get the fixUp searchand replace strings, if any
print "Fix Up Searchand replace needed (1=yes,anything else=no)?";
chomp($line = <STDIN>);
if($line eq "1") {
	print "Fix Up Search? ";
	chomp($line = <STDIN>);
	$inf_data{fixUpSearch}="$line";
# Get the fixUp replace string, if any
	print "Fix Up Replace? ";
	chomp($line = <STDIN>);
	$inf_data{fixUpReplace}="$line";
}

# Get the referer, if any
print "Referer needed (1=yes,anything else=no)?";
chomp($line = <STDIN>);
if($line eq "1") {
	print "Referer? ";
	chomp($line = <STDIN>);
	if ( !($line=~m#^http://[a-zA-Z0-9\~\%\/\.-]*#)) {
		print "malformed URL, start over.\n";
		exit;
	}
	$inf_data{referer}="$line";
}
while ( ($key,$value) = each %inf_data ) {
	if ($value =~ /^([-\@\?\=\~\w.]+)$/) {
		$inf_data{$key} = $1; 			# $data now untainted
	}
}
if ($inf_name =~ /^([-\@\?\=\~\w.]+)$/) {
	$inf_name = $1; 			# $data now untainted
}

$config = new Config::General(\%inf_data);
$config->save_file($inf_name);
}

1;
